Attribute VB_Name = "modGradient"
'*****************************************************************************************
'* Draw a gradient between 2 colors on an object (using Win32 API for best performance): *
'*****************************************************************************************

Option Explicit

Private Type RECT

  Left   As Long
  Top    As Long
  Right  As Long
  Bottom As Long
  
End Type

Private Declare Function GetClientRect Lib "user32" (ByVal hwnd As Long, lpRect As RECT) As Long
Private Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function FillRect Lib "user32" (ByVal hdc As Long, lpRect As RECT, ByVal hBrush As Long) As Long
Private Declare Function SetRect Lib "user32" (lpRect As RECT, ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Long

Private Function SafeDiv(X1 As Double, X2 As Double) As Double

  If X2 = 0 Then SafeDiv = 0 Else SafeDiv = X1 / X2
  
End Function

Public Sub DrawGradientH(obj As Object, Red1 As Integer, Green1 As Integer, Blue1 As Integer, Red2 As Integer, Green2 As Integer, Blue2 As Integer)

  Dim WinRect    As RECT
  Dim ColorRect  As RECT
  Dim X          As Long
  Dim hBrush     As Long
  Dim hPrevBrush As Long
  Dim DivValue   As Double
  Dim CurrRed    As Integer
  Dim CurrGreen  As Integer
  Dim CurrBlue   As Integer
  
  GetClientRect obj.hwnd, WinRect
  
  For X = WinRect.Left To WinRect.Right
  
    DivValue = SafeDiv((WinRect.Right - WinRect.Left), (X - WinRect.Left))
    'Calculate the Red, Green and Blue values:
    CurrRed = Red1 + SafeDiv((Red2 - Red1), DivValue)
    CurrGreen = Green1 + SafeDiv((Green2 - Green1), DivValue)
    CurrBlue = Blue1 + SafeDiv((Blue2 - Blue1), DivValue)
    SetRect ColorRect, X, WinRect.Top, X + 1, WinRect.Bottom
    'Create the brush for the current color:
    hBrush = CreateSolidBrush(RGB(CurrRed, CurrGreen, CurrBlue))
    'Select the brush into the DC:
    hPrevBrush = SelectObject(obj.hdc, hBrush)
    'Draw the line of gradient:
    FillRect obj.hdc, ColorRect, hBrush
    SelectObject obj.hdc, hPrevBrush
    'Release the brush:
    DeleteObject hBrush
    
  Next
  
  obj.Refresh
  
End Sub

Public Sub DrawGradientV(obj As Object, Red1 As Integer, Green1 As Integer, Blue1 As Integer, Red2 As Integer, Green2 As Integer, Blue2 As Integer)

  Dim WinRect    As RECT
  Dim ColorRect  As RECT
  Dim Y          As Long
  Dim hBrush     As Long
  Dim hPrevBrush As Long
  Dim DivValue   As Double
  Dim CurrRed    As Integer
  Dim CurrGreen  As Integer
  Dim CurrBlue   As Integer
  
  GetClientRect obj.hwnd, WinRect
  
  For Y = WinRect.Top To WinRect.Bottom
  
    DivValue = SafeDiv((WinRect.Bottom - WinRect.Top), (Y - WinRect.Top))
    'Calculate the Red, Green and Blue values:
    CurrRed = Red1 + SafeDiv((Red2 - Red1), DivValue)
    CurrGreen = Green1 + SafeDiv((Green2 - Green1), DivValue)
    CurrBlue = Blue1 + SafeDiv((Blue2 - Blue1), DivValue)
    SetRect ColorRect, WinRect.Left, Y, WinRect.Right, Y + 1
    'Create the brush for the current color:
    hBrush = CreateSolidBrush(RGB(CurrRed, CurrGreen, CurrBlue))
    'Select the brush into the DC:
    hPrevBrush = SelectObject(obj.hdc, hBrush)
    'Draw the line of gradient:
    FillRect obj.hdc, ColorRect, hBrush
    SelectObject obj.hdc, hPrevBrush
    'Release the brush:
    DeleteObject hBrush
    
  Next
  
  obj.Refresh
  
End Sub
