Attribute VB_Name = "modResizeKeepRatio"
'****************************************************************************************
'* Module to resize a hWnd at run-time preserving the aspect ratio (using subclassing): *
'****************************************************************************************

Option Explicit

Private defWindowProc As Long
Private AspectRatio   As Single

Private Const GWL_WNDPROC As Long = (-4)
Private Const WM_DESTROY = &H2
Private Const WM_SIZING = &H214

'wParam for WM_SIZING message:
Private Const WMSZ_LEFT = 1
Private Const WMSZ_RIGHT = 2
Private Const WMSZ_TOP = 3
Private Const WMSZ_TOPLEFT = 4
Private Const WMSZ_TOPRIGHT = 5
Private Const WMSZ_BOTTOM = 6
Private Const WMSZ_BOTTOMLEFT = 7
Private Const WMSZ_BOTTOMRIGHT = 8

Private Const PI As Double = 3.14159265358979

Private Type RECT

  Left   As Long
  Top    As Long
  Right  As Long
  Bottom As Long
  
End Type

Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (hpvDest As Any, hpvSource As Any, ByVal cbCopy As Long)

Public Sub Unhook(fhwnd As Long)

  On Error Resume Next
  
  If defWindowProc Then
  
    SetWindowLong fhwnd, GWL_WNDPROC, defWindowProc
    defWindowProc = 0
    
  End If
  
End Sub

Public Sub Hook(fhwnd As Long)

  On Error Resume Next
  defWindowProc = SetWindowLong(fhwnd, GWL_WNDPROC, AddressOf WindowProc)
  
End Sub

Function WindowProc(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

  On Error Resume Next
  
  Dim rc As RECT
  Dim lReturn As Long
  Dim NewRect As RECT
  
  If bResizing = False Then AspectRatio = 0
  
  Select Case uMsg
  
  Case WM_SIZING
  
    'Copy the RECT pointed to in lParam into a RECT structure:
    CopyMemory rc, ByVal lParam, LenB(rc)
    
    'wParam tells which one of the eight possible resizing handles is being used.
    'Set the appropriate RECT member to the size required to maintain aspect ratio, and copy back into the RECT struct for processing by Windows.
    Select Case wParam
    
    Case WMSZ_LEFT
    
      If Not (bSnap2Grid) And rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
      If bSnap2Grid And (iGridSize <> 0) Then
      
        If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) <= 2 Then
        
          rc.Left = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Left - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)
          
        Else
        
          rc.Left = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Left - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)
          
        End If
        
      End If
      
    Case WMSZ_RIGHT
    
      If Not (bSnap2Grid) And rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
      If bSnap2Grid And (iGridSize <> 0) Then
      
        If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) <= 2 Then
        
          rc.Right = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Right - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)
          
        Else
        
          rc.Right = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Right - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)
          
        End If
        
      End If
      
    Case WMSZ_TOP
    
      If Not (bSnap2Grid) And rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
      If bSnap2Grid And (iGridSize <> 0) Then
      
        If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) <= 2 Then
        
          rc.Top = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Top - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)
          
        Else
        
          rc.Top = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Top - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)
          
        End If
        
      End If
      
    Case WMSZ_BOTTOM
    
      If Not (bSnap2Grid) And rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      
      If bSnap2Grid And (iGridSize <> 0) Then
      
        If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) <= 2 Then
        
          rc.Bottom = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Bottom - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)
          
        Else
        
          rc.Bottom = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Bottom - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)
          
        End If
        
      End If
      
    Case WMSZ_TOPLEFT
    
      If (AspectRatio = 0) And (rc.Bottom <> rc.Top) Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      If Not (bSnap2Grid) Then rc.Left = (rc.Top - rc.Bottom) * AspectRatio + rc.Right
      
      If bSnap2Grid And (iGridSize <> 0) Then
      
        If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) <= 2 Then
        
          rc.Top = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Top - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)
          rc.Left = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Left - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)
          
        Else
        
          rc.Top = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Top - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)
          rc.Left = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Left - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)
          
        End If
        
      End If
      
    Case WMSZ_TOPRIGHT
    
      If (AspectRatio = 0) And (rc.Bottom <> rc.Top) Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      If Not (bSnap2Grid) Then rc.Right = (rc.Bottom - rc.Top) * AspectRatio + rc.Left
      
      If bSnap2Grid And (iGridSize <> 0) Then
      
        If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) <= 2 Then
        
          rc.Top = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Top - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)
          rc.Right = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Right - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)
          
        Else
        
          rc.Top = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Top - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)
          rc.Right = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Right - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)
          
        End If
        
      End If
      
    Case WMSZ_BOTTOMLEFT
    
      If (AspectRatio = 0) And (rc.Bottom <> rc.Top) Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      If Not (bSnap2Grid) And (AspectRatio <> 0) Then rc.Bottom = (rc.Right - rc.Left) / AspectRatio + rc.Top
      
      If bSnap2Grid And (iGridSize <> 0) Then
      
        If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) <= 2 Then
        
          rc.Bottom = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Bottom - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)
          rc.Left = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Left - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)
          
        Else
        
          rc.Bottom = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Bottom - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)
          rc.Left = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Left - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)
          
        End If
        
      End If
      
    Case WMSZ_BOTTOMRIGHT
    
      If (AspectRatio = 0) And (rc.Bottom <> rc.Top) Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
      If Not (bSnap2Grid) And (AspectRatio <> 0) Then rc.Bottom = (rc.Right - rc.Left) / AspectRatio + rc.Top
      
      If bSnap2Grid And (iGridSize <> 0) Then
      
        If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) <= 2 Then
        
          rc.Bottom = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Bottom - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoom)) / iTwY)
          rc.Right = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Right - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)) / (iGridSize * fZoom)) * iGridSize * fZoom + Round(((frmPixbook.frmPage(iPage).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoom)) / iTwX)
          
        Else
        
          rc.Bottom = frmPixbook.Top / iTwY + (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Top / iTwY + 2 + Round((rc.Bottom - frmPixbook.Top / iTwY - (frmPixbook.Height - frmPixbook.ScaleHeight) / iTwY + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Top / iTwY - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleHeight / 2) Mod (iGridSize * iTwY * fZoomCvr)) / iTwY)
          rc.Right = frmPixbook.Left / iTwX + (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) + frmPixbook.picPages.Left / iTwX + 2 + Round((rc.Right - frmPixbook.Left / iTwX - (frmPixbook.Width - frmPixbook.ScaleWidth) / (2 * iTwX) - frmPixbook.picPages.Left / iTwX - 2 - Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)) / (iGridSize * fZoomCvr)) * iGridSize * fZoomCvr + Round(((frmPixbook.frmCvr(iCover).ScaleWidth / 2) Mod (iGridSize * iTwX * fZoomCvr)) / iTwX)
          
        End If
        
      End If
      
    End Select
    
    CopyMemory ByVal lParam, rc, LenB(rc)
    WindowProc = 1
'    If rc.Bottom <> rc.Top Then AspectRatio = (rc.Right - rc.Left) / (rc.Bottom - rc.Top)
    HelpLinesMove frmPixbook.picDyn
    RefreshPage
    RefreshCover
    
  Case WM_DESTROY:
  
    'Kill subclassing if active:
    If defWindowProc <> 0 Then defWindowProc = 0
    
  End Select
  
  'Process windows messages:
  WindowProc = CallWindowProc(defWindowProc, hWnd, uMsg, wParam, lParam)
  
End Function
